import wizzi_utils.misc as mt
from wizzi_utils.open_cv.models_dnn_meta_data import Jobs
from wizzi_utils.open_cv.models_dnn_meta_data import DownloadStyle
from wizzi_utils.open_cv.labels_bank import COCO_80_RAW

# from wizzi_utils.open_cv.labels_bank import COCO_YOLO_80_LABELS
# from wizzi_utils.open_cv.models_dnn_meta_data import DownloadStyle
# from wizzi_utils.open_cv.labels_bank import PASCAL_VOC_2012_21_LABELS
# from wizzi_utils.open_cv.labels_bank import PASCAL_VOC_2012_20_LABELS
# from wizzi_utils.open_cv.labels_bank import ILSVRC2016_201_LABELS
# from wizzi_utils.open_cv.labels_bank import COCO_183_LABELS
# from wizzi_utils.open_cv.labels_bank import COCO_182_LABELS
# from wizzi_utils_test.downloaded_models.labels_bank import IMAGENET


# TODO check rpi guide
# https://www.tensorflow.org/lite/guide/python
# to move to mvs virtual env:
# cp -r /usr/lib/python3/dist-packages/tflite_runtime* ~/.virtualenvs/mvs/lib/python3.7/site-packages/
# TODO op1:
# $ wget "https://raw.githubusercontent.com/PINTO0309/TensorflowLite-bin/main/2.5.0/
#           download_tflite_runtime-2.5.0-cp37-none-linux_armv7l.whl.sh"
# $ ./download_tflite_runtime-2.5.0-cp37-none-linux_armv7l.whl.sh
# $ sudo pip3 install --upgrade tflite_runtime-2.5.0-cp37-none-linux_armv7l.whl
# from tflite_runtime.interpreter import Interpreter
# interpreter = Interpreter(model_path="foo.tflite", num_threads=4)

# https://github.com/EdjeElectronics/TensorFlow-Lite-Object-Detection-on-Android-and-Raspberry-Pi/blob/master/Raspberry_Pi_Guide.md
# https://blog.paperspace.com/tensorflow-lite-raspberry-pi/

# TODO more models:
# TODO: https://www.tensorflow.org/lite/guide/hosted_models
# TODO: https://github.com/tensorflow/models/blob/master/research/
#           object_detection/g3doc/tf1_detection_zoo.md#mobile-models  # m1-m4
# TODO: https://www.tensorflow.org/lite/guide/https://github.com/PINTO0309/PINTO_model_zoo
# ssd_mobilenet_coco -
# quick access to tflite models that were designed for devices such as raspberry pi, android and so on
# as far as i know all are ssd_mobilenet models that were trained on coco data set
test_fps_od_image_url = 'https://raw.githubusercontent.com/pjreddie/darknet/master/data/dog.jpg'
test_fps_od_iters = 10
test_fps_computer_info = 'AMD64, Intel64 Family 6 Model 158 Stepping 9, GenuineIntel, ' + \
                         'Physical cores 4, Total cores 8, Frequency 3601.00Mhz, CPU Usage 13.6%)'

fps_info = '({} iterations, the above cfg, on image {} on {})'.format(test_fps_od_iters, test_fps_od_image_url,
                                                                      test_fps_computer_info)

MODELS_TFL_META_DATA = {
    # see wu.tflt.get_all_object_detection_models_info(ack=True)
    'ssd_mobilenet_v3_small_coco_2020_01_14': {
        'job': Jobs.OBJECT_DETECTION.value,
        'default_threshold': 0.2,
        'default_nms': {
            'score_threshold': 0.4,
            'nms_threshold': 0.4,
        },
        'labels_dict': COCO_80_RAW,  # TODO labels are not correct
        'URL': {
            'tflite': 'http://download.tensorflow.org/models/object_detection/' +
                      'ssd_mobilenet_v3_small_coco_2020_01_14.tar.gz',
            'tflite_download_style': DownloadStyle.Tar.value,
            'tflite_name': 'model.tflite',
            'info': 'https://github.com/tensorflow/models/blob/master/research/object_detection/g3doc/'
                    + 'tf1_detection_zoo.md#mobile-models'
        },
        'info': {
            'web_name': 'TODO',
            'size': 'tflite({})'.format(mt.add_color('6.86 MB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('21.96 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'ssd_mobilenet_v3_large_coco_2020_01_14': {
        'job': Jobs.OBJECT_DETECTION.value,
        'default_threshold': 0.2,
        'default_nms': {
            'score_threshold': 0.4,
            'nms_threshold': 0.4,
        },
        'labels_dict': COCO_80_RAW,
        'URL': {
            'tflite': 'http://download.tensorflow.org/models/object_detection/' +
                      'ssd_mobilenet_v3_large_coco_2020_01_14.tar.gz',
            'tflite_download_style': DownloadStyle.Tar.value,
            'tflite_name': 'model.tflite',
            'info': 'https://github.com/tensorflow/models/blob/master/research/object_detection/g3doc/'
                    + 'tf1_detection_zoo.md#mobile-models'
        },
        'info': {
            'web_name': 'TODO',
            'size': 'tflite({})'.format(mt.add_color('12.42 MB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('12.94 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'ssd_mobilenet_v2_mnasfpn': {
        'job': Jobs.OBJECT_DETECTION.value,
        'default_threshold': 0.2,
        'default_nms': {
            'score_threshold': 0.4,
            'nms_threshold': 0.4,
        },
        'labels_dict': COCO_80_RAW,
        'URL': {
            'tflite': 'http://download.tensorflow.org/models/object_detection/' +
                      'ssd_mobilenet_v2_mnasfpn_shared_box_predictor_320x320_coco_sync_2020_05_18.tar.gz',
            'tflite_download_style': DownloadStyle.Tar.value,
            'tflite_name': 'model.tflite',
            'info': 'https://github.com/tensorflow/models/blob/master/research/object_detection/g3doc/'
                    + 'tf1_detection_zoo.md#mobile-models'
        },
        'info': {
            'web_name': 'TODO',
            'size': 'tflite({})'.format(mt.add_color('9.68 MB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('5.65 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'ssdlite_mobiledet_cpu_320x320_coco_2020_05_19': {
        'job': Jobs.OBJECT_DETECTION.value,
        'default_threshold': 0.2,
        'default_nms': {
            'score_threshold': 0.4,
            'nms_threshold': 0.4,
        },
        'labels_dict': COCO_80_RAW,
        'URL': {
            'tflite': 'http://download.tensorflow.org/models/object_detection/' +
                      'ssdlite_mobiledet_cpu_320x320_coco_2020_05_19.tar.gz',
            'tflite_download_style': DownloadStyle.Tar.value,
            'tflite_name': 'model.tflite',
            'info': 'https://github.com/tensorflow/models/blob/master/research/object_detection/g3doc/'
                    + 'tf1_detection_zoo.md#mobile-models'
        },
        'info': {
            'web_name': 'TODO',
            'size': 'tflite({})'.format(mt.add_color('15.97 MB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('11.01 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'ssd_mobilenet_v1_1_metadata_1': {
        'job': Jobs.OBJECT_DETECTION.value,
        'default_threshold': 0.2,
        'default_nms': {
            'score_threshold': 0.4,
            'nms_threshold': 0.4,
        },
        'labels_dict': COCO_80_RAW,
        'URL': {
            'tflite': 'https://tfhub.dev/tensorflow/lite-model/ssd_mobilenet_v1/1/metadata/1?lite-format=tflite',
            'tflite_download_style': DownloadStyle.Direct.value,
            'info': 'https://www.tensorflow.org/lite/examples/object_detection/overview'
        },
        'info': {
            'web_name': 'TODO',
            'size': 'tflite({})'.format(mt.add_color('3.99 MB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('4.43 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'ssdlite_mobilenet_v2_coco_300_integer_quant_with_postprocess': {
        'job': Jobs.OBJECT_DETECTION.value,
        'default_threshold': 0.2,
        'default_nms': {
            'score_threshold': 0.4,
            'nms_threshold': 0.4,
        },
        'labels_dict': COCO_80_RAW,
        'URL': {
            'tflite': 'https://drive.google.com/' +
                      'uc?export=download&confirm=${CODE}&id=1LjTqn5nChAVKhXgwBUp00XIKXoZrs9sB',
            'tflite_download_style': DownloadStyle.Direct.value,
            'info': 'https://github.com/PINTO0309/PINTO_model_zoo/tree/main/006_mobilenetv2-ssdlite/01_coco/'
                    + '03_integer_quantization'
        },
        'info': {
            'web_name': 'TODO',
            'size': 'tflite({})'.format(mt.add_color('5.09 MB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('0.63 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'coco_ssd_mobilenet_v1_1_0_quant_2018_06_29': {
        'job': Jobs.OBJECT_DETECTION.value,
        'default_threshold': 0.2,
        'default_nms': {
            'score_threshold': 0.4,
            'nms_threshold': 0.4,
        },
        'labels_dict': COCO_80_RAW,
        'URL': {
            'tflite': 'http://storage.googleapis.com/download.tensorflow.org/models/tflite/' +
                      'coco_ssd_mobilenet_v1_1.0_quant_2018_06_29.zip',
            'tflite_download_style': DownloadStyle.Zip.value,
            'tflite_name': 'detect.tflite',
            'info': 'https://gist.github.com/iwatake2222/e4c48567b1013cf31de1cea36c4c061c'
        },
        'info': {
            'web_name': 'TODO',
            'size': 'tflite({})'.format(mt.add_color('3.99 MB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('5.10 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    # 'pose_detection': {  # NOT TESTED
    #     'job': Jobs.OBJECT_DETECTION.value,
    #     'threshold': 0.2,
    #     'joint_names': [],
    #     'URL': {
    #         'tflite': 'https://github.com/google/mediapipe/raw/master/mediapipe/modules/pose_detection/' +
    #                   'pose_detection.tflite',
    #         'tflite_download_style': DownloadStyle.Direct.value,
    #         'info': 'https://github.com/google/mediapipe/tree/master/mediapipe/modules/pose_detection'
    #     },
    #     'info': {
    #         'web_name': 'TODO',
    #         'size': 'tflite({})'.format(mt.add_color('xxxx MB', ops='c')),
    #         'fps': '{} {}'.format(mt.add_color('xxxx FPS'), fps_info),
    #         'info': 'TODO',
    #     },
    # },
    'pose_landmark_lite': {
        'job': Jobs.OPEN_POSE.value,
        'threshold': 0.2,
        'joint_names': [
            'nose',  # 0
            'leftEyeInside',  # 1
            'leftEye',  # 2
            'leftEyeOutside',  # 3
            'rightEyeInside',  # 4
            'rightEye',  # 5
            'rightEyeOutside',  # 6
            'leftEar',  # 7
            'rightEar',  # 8
            'leftMouth',  # 9
            'rightMouth',  # 10
            'leftShoulder',  # 11
            'rightShoulder',  # 12
            'leftElbow',  # 13
            'rightElbow',  # 14
            'leftWrist',  # 15
            'rightWrist',  # 16
            'leftPalm',  # 17
            'rightPalm',  # 18
            'leftIndex',  # 19
            'rightIndex',  # 20
            'leftPinky',  # 21
            'rightPinky',  # 22
            'leftHip',  # 23
            'rightHip',  # 24
            'leftKnee',  # 25
            'rightKnee',  # 26
            'leftAnkle',  # 27
            'rightAnkle',  # 28
            'leftHeel',  # 29
            'rightHeel',  # 30
            'leftFoot',  # 31
            'rightFoot',  # 32
            # 'midHip',  # 33  # dont think it's supported
            # 'forehead',  # 34
            # 'leftThumb',  # 35
            # 'leftHand',  # 36
            # 'rightThumb',  # 37
            # 'rightHand',  # 38
        ],
        'connect_groups': [
            ['leftEar', 'leftEyeOutside', 'leftEye', 'leftEyeInside', 'nose', 'leftMouth', 'rightMouth', 'nose',
             'rightEyeInside', 'rightEye', 'rightEyeOutside', 'rightEar'],
            ['leftPalm', 'leftWrist', 'leftElbow', 'leftShoulder'],
            ['rightPalm', 'rightWrist', 'rightElbow', 'rightShoulder'],
            ['leftHip', 'leftKnee', 'leftAnkle', 'leftHeel', 'leftFoot'],
            ['rightHip', 'rightKnee', 'rightAnkle', 'rightHeel', 'rightFoot'],
            ['leftHip', 'rightHip', 'rightShoulder', 'leftShoulder', 'leftHip']
        ],
        'URL': {
            'tflite': 'https://github.com/google/mediapipe/raw/master/mediapipe/modules/pose_landmark/' +
                      'pose_landmark_lite.tflite',
            'tflite_download_style': DownloadStyle.Direct.value,
            'info': 'https://github.com/google/mediapipe/tree/master/mediapipe/modules/pose_landmark',
            'info2': 'https://github.com/vladmandic/blazepose/tree/fe647445507e37469d96da6fde5c8b0980f745bc'
        },
        'info': {
            'web_name': 'TODO',
            'size': 'tflite({})'.format(mt.add_color('xxxx MB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('xxxx FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'pose_landmark_full': {
        'job': Jobs.OPEN_POSE.value,
        'threshold': 0.2,
        'joint_names': [
            'nose',  # 0
            'leftEyeInside',  # 1
            'leftEye',  # 2
            'leftEyeOutside',  # 3
            'rightEyeInside',  # 4
            'rightEye',  # 5
            'rightEyeOutside',  # 6
            'leftEar',  # 7
            'rightEar',  # 8
            'leftMouth',  # 9
            'rightMouth',  # 10
            'leftShoulder',  # 11
            'rightShoulder',  # 12
            'leftElbow',  # 13
            'rightElbow',  # 14
            'leftWrist',  # 15
            'rightWrist',  # 16
            'leftPalm',  # 17
            'rightPalm',  # 18
            'leftIndex',  # 19
            'rightIndex',  # 20
            'leftPinky',  # 21
            'rightPinky',  # 22
            'leftHip',  # 23
            'rightHip',  # 24
            'leftKnee',  # 25
            'rightKnee',  # 26
            'leftAnkle',  # 27
            'rightAnkle',  # 28
            'leftHeel',  # 29
            'rightHeel',  # 30
            'leftFoot',  # 31
            'rightFoot',  # 32
            # 'midHip',  # 33  # dont think it's supported
            # 'forehead',  # 34
            # 'leftThumb',  # 35
            # 'leftHand',  # 36
            # 'rightThumb',  # 37
            # 'rightHand',  # 38
        ],
        'connect_groups': [
            ['leftEar', 'leftEyeOutside', 'leftEye', 'leftEyeInside', 'nose', 'leftMouth', 'rightMouth', 'nose',
             'rightEyeInside', 'rightEye', 'rightEyeOutside', 'rightEar'],
            ['leftPalm', 'leftWrist', 'leftElbow', 'leftShoulder'],
            ['rightPalm', 'rightWrist', 'rightElbow', 'rightShoulder'],
            ['leftHip', 'leftKnee', 'leftAnkle', 'leftHeel', 'leftFoot'],
            ['rightHip', 'rightKnee', 'rightAnkle', 'rightHeel', 'rightFoot'],
            ['leftHip', 'rightHip', 'rightShoulder', 'leftShoulder', 'leftHip']
        ],
        'URL': {
            'tflite': 'https://github.com/google/mediapipe/raw/master/mediapipe/modules/pose_landmark/' +
                      'pose_landmark_full.tflite',
            'tflite_download_style': DownloadStyle.Direct.value,
            'info': 'https://github.com/google/mediapipe/tree/master/mediapipe/modules/pose_detection',
            'info2': 'https://github.com/vladmandic/blazepose/tree/fe647445507e37469d96da6fde5c8b0980f745bc'
        },
        'info': {
            'web_name': 'TODO',
            'size': 'tflite({})'.format(mt.add_color('xxxx MB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('xxxx FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'pose_landmark_heavy': {
        'job': Jobs.OPEN_POSE.value,
        'threshold': 0.2,
        'joint_names': [
            'nose',  # 0
            'leftEyeInside',  # 1
            'leftEye',  # 2
            'leftEyeOutside',  # 3
            'rightEyeInside',  # 4
            'rightEye',  # 5
            'rightEyeOutside',  # 6
            'leftEar',  # 7
            'rightEar',  # 8
            'leftMouth',  # 9
            'rightMouth',  # 10
            'leftShoulder',  # 11
            'rightShoulder',  # 12
            'leftElbow',  # 13
            'rightElbow',  # 14
            'leftWrist',  # 15
            'rightWrist',  # 16
            'leftPalm',  # 17
            'rightPalm',  # 18
            'leftIndex',  # 19
            'rightIndex',  # 20
            'leftPinky',  # 21
            'rightPinky',  # 22
            'leftHip',  # 23
            'rightHip',  # 24
            'leftKnee',  # 25
            'rightKnee',  # 26
            'leftAnkle',  # 27
            'rightAnkle',  # 28
            'leftHeel',  # 29
            'rightHeel',  # 30
            'leftFoot',  # 31
            'rightFoot',  # 32
            # 'midHip',  # 33  # dont think it's supported
            # 'forehead',  # 34
            # 'leftThumb',  # 35
            # 'leftHand',  # 36
            # 'rightThumb',  # 37
            # 'rightHand',  # 38
        ],
        'connect_groups': [
            ['leftEar', 'leftEyeOutside', 'leftEye', 'leftEyeInside', 'nose', 'leftMouth', 'rightMouth', 'nose',
             'rightEyeInside', 'rightEye', 'rightEyeOutside', 'rightEar'],
            ['leftPalm', 'leftWrist', 'leftElbow', 'leftShoulder'],
            ['rightPalm', 'rightWrist', 'rightElbow', 'rightShoulder'],
            ['leftHip', 'leftKnee', 'leftAnkle', 'leftHeel', 'leftFoot'],
            ['rightHip', 'rightKnee', 'rightAnkle', 'rightHeel', 'rightFoot'],
            ['leftHip', 'rightHip', 'rightShoulder', 'leftShoulder', 'leftHip']
        ],
        'URL': {
            'tflite': 'https://github.com/google/mediapipe/raw/master/mediapipe/modules/pose_landmark/' +
                      'pose_landmark_heavy.tflite',
            'tflite_download_style': DownloadStyle.Direct.value,
            'info': 'https://github.com/google/mediapipe/tree/master/mediapipe/modules/pose_detection',
            'info2': 'https://github.com/vladmandic/blazepose/tree/fe647445507e37469d96da6fde5c8b0980f745bc'
        },
        'info': {
            'web_name': 'TODO',
            'size': 'tflite({})'.format(mt.add_color('xxxx MB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('xxxx FPS'), fps_info),
            'info': 'TODO',
        },
    },
}



from wizzi_utils.misc import misc_tools as mt
from wizzi_utils.misc.test import test_misc_tools as mtt
from wizzi_utils.open_cv import open_cv_tools as cvt
from wizzi_utils.socket import socket_tools as st
from wizzi_utils.pyplot import pyplot_tools as pyplt
import numpy as np
import os
# noinspection PyPackageRequirements
import cv2

LOOP_TESTS = 50
BLOCK_MS_NORMAL = 2000  # 0 to block
ITERS_CAM_TEST = 10  # 0 to block
MODEL_FRAMES_CAM = 3
SAVE_SCOPE = 'CVT'
OD_BEST_MODEL = 'yolov4'
# POSE_BEST_MODEL = 'openpose_pose_coco'
# POSE_BEST_MODEL = 'openpose_pose_mpi'
# POSE_BEST_MODEL = 'hand_pose'
POSE_BEST_MODEL = 'body25'
BEST_MODELS = 'best_models'
ALL_MODELS = 'all_models'


def load_img_from_web(name: str) -> np.array:
    f = mtt.IMAGES_INPUTS
    url = mtt.IMAGES_D[name]
    suffix = 'jpg'  # default
    # if '.webm' in url:
    #     suffix = 'webm'
    dst = '{}/{}.{}'.format(f, name, suffix)

    if not os.path.exists(dst):
        if not os.path.exists(f):
            mt.create_dir(f)
        success = st.download_file(url, dst)
        if not success:
            mt.exception_error('download failed - creating random img', real_exception=False)
            img = mt.np_random_integers(size=(240, 320, 3), low=0, high=255)
            img = img.astype('uint8')
            cvt.save_img(dst, img)

    img = cvt.load_img(path=dst)
    return img


def get_vid_from_web(name: str) -> str:
    f = mtt.VIDEOS_INPUTS
    url = mtt.VIDEOS_D[name]
    suffix = 'mp4'  # default
    if '.webm' in url:
        suffix = 'webm'
    dst = '{}/{}.{}'.format(f, name, suffix)

    if not os.path.exists(dst):
        if not os.path.exists(f):
            mt.create_dir(f)
        success = st.download_file(url, dst)
        if not success:
            mt.exception_error('download failed - creating random img', real_exception=False)
            dst = None

    return dst


def get_cv_version_test():
    mt.get_function_name(ack=True, tabs=0)
    cvt.get_cv_version(ack=True, tabs=1)
    return


def imread_imwrite_test():
    mt.get_function_name(ack=True, tabs=0)
    name = mtt.SO_LOGO
    img = load_img_from_web(name)

    f = mtt.IMAGES_INPUTS
    url = mtt.IMAGES_D[name]
    dst_path = '{}/{}'.format(f, os.path.basename(url).replace('.png', '_copy.png'))

    cvt.save_img(dst_path, img, ack=True)
    img_loaded = cvt.load_img(dst_path, ack=True)
    print(mt.to_str(img_loaded, '\timg_copy'))
    mt.delete_file(dst_path, ack=True)
    # mt.delete_file(file=mtt.SO_LOGO_PATH, ack=True)
    return


def list_to_cv_image_test():
    mt.get_function_name(ack=True, tabs=0)
    img = load_img_from_web(mtt.SO_LOGO)
    img_list = img.tolist()
    print(mt.to_str(img_list, '\timg_list'))
    img = cvt.list_to_cv_image(img_list)
    print(mt.to_str(img, '\timg'))
    # mt.delete_file(file=mtt.TEMP_IMAGE_PATH, ack=True)
    return


def display_open_cv_image_test():
    mt.get_function_name(ack=True, tabs=0)
    img = load_img_from_web(mtt.SO_LOGO)
    print('\tVisual test: stack overflow logo')
    loc = (70, 200)  # move to X,Y
    resize = 1.7  # enlarge to 170%
    cvt.display_open_cv_image(
        img=img,
        ms=1,  # not blocking
        title='stack overflow logo moved to {} and re-sized to {}'.format(loc, resize),
        loc=loc,  # start from x =70 y = 0
        resize=resize
    )
    loc = pyplt.Location.TOP_RIGHT.value  # move to top right corner
    resize = 1.7  # enlarge to 170%
    cvt.display_open_cv_image(
        img=img,
        ms=BLOCK_MS_NORMAL,  # blocking
        title='stack overflow logo moved to {} and re-sized to {}'.format(loc, resize),
        loc=loc,  # start from x =70 y = 0
        resize=resize
    )
    cv2.destroyAllWindows()
    # mt.delete_file(file=mtt.SO_LOGO_PATH, ack=True)
    return


def display_open_cv_image_loop_test():
    mt.get_function_name(ack=True, tabs=0)
    img = load_img_from_web(mtt.SO_LOGO)
    loc = (70, 200)  # move to X,Y
    resize = 1.7  # enlarge to 170%
    title = 'stack overflow logo moved to {} and re-sized to {} - {} iterations'.format(loc, resize, LOOP_TESTS)
    print('\tVisual test: {}'.format(title))
    for i in range(LOOP_TESTS):
        cvt.display_open_cv_image(
            img=img,
            ms=1,  # not blocking
            title=title,
            loc=loc,  # start from x =70 y = 0
            resize=resize
        )
        if i == 0:  # move just first iter
            loc = None
    cv2.destroyAllWindows()
    # mt.delete_file(file=mtt.SO_LOGO_PATH, ack=True)
    return


def resize_opencv_image_test():
    mt.get_function_name(ack=True, tabs=0)
    img = load_img_from_web(mtt.SO_LOGO)
    print(mt.to_str(img, '\timg'))
    img = cvt.resize_opencv_image(img, scale_percent=0.6)
    print(mt.to_str(img, '\timg re-sized to 60%'))
    # mt.delete_file(file=mtt.SO_LOGO_PATH, ack=True)
    return


def move_cv_img_x_y_test():
    mt.get_function_name(ack=True, tabs=0)
    img = load_img_from_web(mtt.SO_LOGO)
    options = [(0, 0), (100, 0), (0, 100), (150, 150), (400, 400), (250, 350)]
    print('\tVisual test: move to all options {}'.format(options))
    print('\t\tClick Esc to close all')
    for x_y in options:
        title = 'move to ({})'.format(x_y)
        cv2.imshow(title, img)
        cvt.move_cv_img_x_y(title, x_y)
    cv2.waitKey(BLOCK_MS_NORMAL)
    cv2.destroyAllWindows()
    # mt.delete_file(file=mtt.SO_LOGO_PATH, ack=True)
    return


def move_cv_img_by_str_test():
    mt.get_function_name(ack=True, tabs=0)
    img = load_img_from_web(mtt.SO_LOGO)
    options = pyplt.Location.get_location_list_by_rows()
    print('\tVisual test: move to all options {}'.format(options))
    print('\t\tClick Esc to close all')
    for where_to in options:
        title = 'move to {}'.format(where_to)
        cv2.imshow(title, img)
        cvt.move_cv_img_by_str(img, title, where=where_to)
    cv2.waitKey(BLOCK_MS_NORMAL)
    cv2.destroyAllWindows()
    # mt.delete_file(file=mtt.SO_LOGO_PATH, ack=True)
    return


def unpack_list_imgs_to_big_image_test():
    mt.get_function_name(ack=True, tabs=0)
    img = load_img_from_web(mtt.SO_LOGO)
    gray = cvt.BGR_img_to_gray(img)
    big_img = cvt.unpack_list_imgs_to_big_image(
        imgs=[img, gray, img],
        resize=None,
        grid=(2, 2)
    )
    title = 'stack overflow logo 2x2(1 empty)'
    print('\tVisual test: {}'.format(title))
    cvt.display_open_cv_image(
        img=big_img,
        ms=BLOCK_MS_NORMAL,  # blocking
        title=title,
        loc=(0, 0),
        resize=None
    )
    cv2.destroyAllWindows()
    # mt.delete_file(file=mtt.SO_LOGO_PATH, ack=True)
    return


def display_open_cv_images_test():
    mt.get_function_name(ack=True, tabs=0)
    img = load_img_from_web(mtt.SO_LOGO)
    title = '2x1 grid'
    print('\tVisual test: {}'.format(title))
    loc1 = (0, 0)
    cvt.display_open_cv_images(
        imgs=[img, img],
        ms=1,  # blocking
        title='{} loc={}'.format(title, loc1),
        loc=loc1,
        resize=None,
        grid=(2, 1),
        header='{} loc={}'.format(title, loc1),
    )
    loc2 = pyplt.Location.BOTTOM_CENTER.value
    cvt.display_open_cv_images(
        imgs=[img, img],
        ms=BLOCK_MS_NORMAL,  # blocking
        title='{} loc={}'.format(title, loc2),
        loc=loc2,
        resize=None,
        grid=(2, 1),
        header='{} loc={}'.format(title, loc1),
    )
    cv2.destroyAllWindows()
    # mt.delete_file(file=mtt.SO_LOGO_PATH, ack=True)
    return


def display_open_cv_images_loop_test():
    mt.get_function_name(ack=True, tabs=0)
    img = load_img_from_web(mtt.SO_LOGO)
    loc = (70, 200)  # move to X,Y
    title = 'stack overflow logo moved to {} - {} iterations'.format(loc, LOOP_TESTS)
    print('\tVisual test: {}'.format(title))
    for i in range(LOOP_TESTS):
        cvt.display_open_cv_images(
            imgs=[img, img],
            ms=1,  # blocking
            title=title,
            loc=loc,
            resize=None,
            grid=(2, 1),
            header=None
        )
        if i == 0:  # move just first iter
            loc = None
    cv2.destroyAllWindows()
    # mt.delete_file(file=mtt.SO_LOGO_PATH, ack=True)
    return


def gray_to_BGR_and_back_test():
    mt.get_function_name(ack=True, tabs=0)
    img = load_img_from_web(mtt.SO_LOGO)
    print(mt.to_str(img, '\timgRGB'))
    gray = cvt.BGR_img_to_gray(img)
    print(mt.to_str(img, '\timg_gray'))
    img = cvt.gray_scale_img_to_BGR_form(gray)
    print(mt.to_str(img, '\timgRGB'))
    # mt.delete_file(file=mtt.SO_LOGO_PATH, ack=True)
    return


def BGR_img_to_RGB_and_back_test():
    mt.get_function_name(ack=True, tabs=0)
    imgBGR1 = load_img_from_web(mtt.SO_LOGO)
    print(mt.to_str(imgBGR1, '\timgBGR'))
    imgRGB = cvt.BGR_img_to_RGB(imgBGR1)
    print(mt.to_str(imgRGB, '\timgRGB'))
    imgBGR2 = cvt.RGB_img_to_BGR(imgRGB)
    print(mt.to_str(imgBGR2, '\timgBGR2'))

    cvt.display_open_cv_images(
        imgs=[imgBGR1, imgRGB, imgBGR2],
        ms=BLOCK_MS_NORMAL,  # blocking
        title='imgBGR1, imgRGB, imgBGR2',
        loc=pyplt.Location.CENTER_CENTER,
        resize=None,
        grid=(3, 1),
        header='compare'
    )
    cv2.destroyAllWindows()
    # mt.delete_file(file=mtt.SO_LOGO_PATH, ack=True)
    return


def CameraWu_test(type_cam: str):
    WITH_SLEEP = False
    ports = [0, 1, 13]
    cams = []
    for port in ports:
        cam = cvt.CameraWu.open_camera(port=port, type_cam=type_cam)
        if cam is not None:
            cams.append(cam)

    for cam in cams:
        title = 'CameraWu_test({}) on port {}'.format(cam.type_cam, cam.port)
        fps = mt.FPS(summary_title=title)
        for i in range(ITERS_CAM_TEST):
            fps.start()
            success, cv_img = cam.read_img()
            if WITH_SLEEP:
                mt.sleep(1)

            if success:
                cvt.display_open_cv_image(
                    img=cv_img,
                    ms=1,
                    title=title,
                    loc=pyplt.Location.CENTER_CENTER,
                    resize=None,
                    header='{}/{}'.format(i + 1, ITERS_CAM_TEST)
                )
            fps.update()
        fps.finalize()
    cv2.destroyAllWindows()
    return


def CameraWu_cv2_test():
    mt.get_function_name(ack=True, tabs=0)
    CameraWu_test(type_cam='cv2')
    return


def CameraWu_acapture_test():
    mt.get_function_name(ack=True, tabs=0)
    CameraWu_test(type_cam='acapture')
    return


def CameraWu_imutils_test():
    mt.get_function_name(ack=True, tabs=0)
    CameraWu_test(type_cam='imutils')
    return


def od_fps_models_test():
    iterations = 10
    images_names = [mtt.DOGS1] * iterations
    cv_imgs_orig = [load_img_from_web(image_name) for image_name in images_names]
    print(images_names)
    od_models_compare_images_test(
        cv_imgs_orig=cv_imgs_orig,
        images_names=images_names,
        models_selection=ALL_MODELS,
        ms=1
    )
    return


def __pose_get_models(models_selection: str = BEST_MODELS):
    if models_selection == BEST_MODELS:
        # preparing 4 best models
        grid = (2, 2)
        display_im_size = (640, 480)
        models_names = [
            'openpose_pose_mpi',
            'openpose_pose_coco',
            'body25',
            'hand_pose',
        ]
    elif models_selection == ALL_MODELS:
        # preparing all(4) models
        grid = (2, 2)
        display_im_size = (640, 480)
        models_names = cvt.get_all_pose_models_info(ack=False)
        pass
    else:
        # preparing 1 models - default is BEST_MODEL
        grid = (1, 1)
        display_im_size = (640, 480)
        models_names = [models_selection]
    models = []
    for m_name in models_names:
        save_dir = '{}/{}/{}'.format(mtt.CV2_MODELS, cvt.MODELS_OPEN_POSE_META_DATA[m_name]['job'], m_name)
        m = cvt.DnnOpenPoseModels(
            save_load_dir=save_dir,
            model_name=m_name,
            tabs=1,
            # threshold=x,  # take default
            # in_dims=(x, x),  # take default
            # scalefactor=x,  # take default
            # mean=(0, 0, 0),  # take default
            # swapRB=x,  # take default
            # crop=x,  # take default
        )
        print(m)
        models.append(m)
    return models, grid, display_im_size


def __od_get_models(models_selection: str = BEST_MODELS):
    if models_selection == BEST_MODELS:
        # preparing 4 best models
        grid = (2, 2)
        display_im_size = (640, 480)
        models_names = [
            'yolov3',
            'yolov3-ssp',
            'yolov4',
            'yolov4-tiny',
        ]
    elif models_selection == ALL_MODELS:
        # preparing all(19) models
        grid = (4, 5)
        display_im_size = (320, 240)
        models_names = cvt.get_all_object_detection_models_info(ack=False)
    else:
        # preparing 1 models - default is BEST_MODEL
        grid = (1, 1)
        display_im_size = (640, 480)
        models_names = [models_selection]
    models = []
    for m_name in models_names:
        save_dir = '{}/{}/{}'.format(mtt.CV2_MODELS, cvt.MODELS_DNN_OBJECT_DETECTION_META_DATA[m_name]['job'], m_name)
        m = cvt.DnnObjectDetectionModels(
            save_load_dir=save_dir,
            model_name=m_name,
            # allowed_class=['dog', 'cat'],
            tabs=1,
            # threshold=0.1,  # take default
            # nms_threshold=0.3,  # take default
            # in_dims=(416, 416),  # take default
            # scalefactor=1 / 127.5,  # take default
            # mean=(0, 0, 0),  # take default
            # swapRB=True,  # take default
            # crop=False,  # take default
        )
        print(m)
        models.append(m)
    return models, grid, display_im_size


def __od_models_images_test(
        models: list,
        images_names: list,
        cv_imgs_orig: list,
        grid: tuple,
        delay_ms: int,
        save_dir: (str, None),
        display_im_size: tuple,
        with_tf: tuple = False,
        with_sub: tuple = False,
):
    """
    AUX FUNCTION
    :param models:
    :param images_names: for debugging (name of classification, save output name ...)
    :param cv_imgs_orig: will classify on this images
    :param grid:
    :param delay_ms:
    :param save_dir:
    :param display_im_size:
    :return:
    """
    # Prepare cv images list
    fps_list = [mt.FPS(summary_title='detection {}'.format(models[i].name)) for i in range(len(models))]
    for cv_img, img_name in zip(cv_imgs_orig, images_names):
        cv_img_per_model = []
        for j, model in enumerate(models):
            cv_img_clone = cv_img.copy()
            fps_list[j].start()
            detections = model.detect_cv_img(
                cv_img=cv_img,
                fp=2,
                ack=False, # TODO False
                tabs=2,
                img_title=img_name
            )
            fps_list[j].update(ack_progress=True, tabs=1, with_title=True)
            if with_tf:
                detections = model.add_traffic_light_to_detections(
                    detections,
                    traffic_light_p={
                        'up': 0.2,
                        'mid': 0.3,
                        'down': 0.4
                    }
                )
            if with_sub:
                detections = model.add_sub_sub_image_to_detection(
                    detections,
                    cv_img=cv_img,
                    bbox_image_p={
                        'x_start': 0.2,
                        'x_end': 0.8,
                        'y_start': 1,
                        'y_end': 0.5,
                    },
                )

            model.draw_detections(
                detections,
                # colors_d=tflt.ssd_mobilenet_coco.DEFAULT_COLOR_D,
                colors_d={
                    'bbox': 'r',
                    'label_bbox': 'black',
                    'text': 'white',
                    'sub_image': 'blue',
                    'person_bbox': 'lightgreen',
                    'dog_bbox': 'blue',
                },
                cv_img=cv_img_clone,
                draw_labels=True,
                ack=False,
                tabs=1,
                img_title=img_name,
            )
            title = 'detections({}) on image {} {}:'.format(len(detections), img_name, cv_img.shape)
            cvt.add_header(cv_img_clone, header=title, loc=pyplt.Location.TOP_LEFT.value, bg_font_scale=2)
            cvt.add_header(cv_img_clone, header=model.name, loc=pyplt.Location.BOTTOM_LEFT.value, bg_font_scale=1)
            cv_img_clone = cv2.resize(cv_img_clone, display_im_size, interpolation=cv2.INTER_AREA)
            cv_img_per_model.append(cv_img_clone)

        if save_dir is not None:
            save_path = '{}/{}.jpg'.format(save_dir, img_name)
        else:
            save_path = None

        cvt.display_open_cv_images(
            cv_img_per_model,
            ms=delay_ms,
            title='{}'.format(img_name),
            loc=pyplt.Location.CENTER_CENTER.value,
            resize=None,
            grid=grid,
            header=None,
            save_path=save_path
        )
        cv2.destroyAllWindows()
    for fps in fps_list:
        fps.finalize()
    return


def __od_models_cap_test(
        models: list,
        cap: (cv2.VideoCapture, cvt.CameraWu),
        total_frames: int,
        work_every_x_frames: int,
        grid: tuple,
        delay_ms: int,
        save_dir: (str, None),
        display_im_size: tuple,
        cap_desc: str,
        with_tf: tuple = False,
        with_sub: tuple = False,
):
    """
    AUX FUNCTION
    :param models:
    :param grid:
    :param delay_ms:
    :param display_im_size:
    :return:
    """
    if save_dir is not None:
        out_fp = '{}/{}_detected.mp4'.format(save_dir, cap_desc)
        out_dims = (display_im_size[0] * grid[1], display_im_size[1] * grid[0])
        mp4 = cvt.Mp4_creator(
            out_full_path=out_fp,
            out_fps=20.0,
            out_dims=out_dims
        )
        print(mp4)
        print('\tvid size output will be {}'.format(out_dims))
    else:
        mp4 = None
    fps_list = [mt.FPS(summary_title='detection {}'.format(models[i].name)) for i in range(len(models))]

    for i in range(total_frames):
        if isinstance(cap, cv2.VideoCapture):
            success, cv_img = cap.read()
        else:
            success, cv_img = cap.read_img()
        if i % work_every_x_frames != 0:  # s
            # do only 10 frames
            continue
        print('\tframe {}/{}:'.format(i + 1, total_frames))
        if success:
            cv_img_per_model = []
            for j, model in enumerate(models):
                cv_img_clone = cv_img.copy()
                fps_list[j].start()
                detections = model.detect_cv_img(
                    cv_img=cv_img,
                    fp=2,
                    ack=False,
                    tabs=1,
                    img_title='frame {}'.format(i + 1)
                )
                fps_list[j].update(ack_progress=True, tabs=2, with_title=True)
                if with_tf:
                    detections = model.add_traffic_light_to_detections(
                        detections,
                        traffic_light_p={
                            'up': 0.2,
                            'mid': 0.3,
                            'down': 0.4
                        }
                    )
                if with_sub:
                    detections = model.add_sub_sub_image_to_detection(
                        detections,
                        cv_img=cv_img,
                        bbox_image_p={
                            'x_start': 0.2,
                            'x_end': 0.8,
                            'y_start': 1,
                            'y_end': 0.5,
                        },
                    )

                model.draw_detections(
                    detections,
                    # colors_d=tflt.ssd_mobilenet_coco.DEFAULT_COLOR_D,
                    colors_d={
                        'bbox': 'r',
                        'label_bbox': 'black',
                        'text': 'white',
                        'sub_image': 'blue',
                        'person_bbox': 'lightgreen',
                        'dog_bbox': 'blue',
                    },
                    cv_img=cv_img_clone,
                    draw_labels=True,
                    ack=False,
                    tabs=1,
                    img_title='frame {}'.format(i + 1),
                )
                title = 'detections({}) on frame {}/{} {}:'.format(len(detections), (i + 1), total_frames, cv_img.shape)
                cvt.add_header(cv_img_clone, header=title, loc=pyplt.Location.TOP_LEFT.value, bg_font_scale=2)
                cvt.add_header(cv_img_clone, header=model.name, loc=pyplt.Location.BOTTOM_LEFT.value, bg_font_scale=1)
                cv_img_clone = cv2.resize(cv_img_clone, display_im_size, interpolation=cv2.INTER_AREA)
                cv_img_per_model.append(cv_img_clone)
            if mp4 is not None:
                big_img = cvt.unpack_list_imgs_to_big_image(cv_img_per_model, grid=grid, resize=None)
                mp4.add_frame(big_img)
            cv_title = '{} models'.format(len(models)) if len(models) > 1 else models[0].name
            cv_title += ' on {}'.format(cap_desc)
            cvt.display_open_cv_images(
                cv_img_per_model,
                ms=delay_ms,
                title=cv_title,
                loc=pyplt.Location.CENTER_CENTER.value,
                resize=None,
                grid=grid,
                header=None,
                save_path=None
            )
    for fps in fps_list:
        fps.finalize()
    cv2.destroyAllWindows()
    if mp4 is not None:
        mp4.finalize()
    return


def od_best_model_images_test(m_name: str = OD_BEST_MODEL, ms: int = BLOCK_MS_NORMAL):
    mt.get_function_name(ack=True, tabs=0)
    od_models_compare_images_test(
        models_selection=m_name,
        ms=ms
    )
    return


def od_models_compare_images_test(
        cv_imgs_orig: list = None,
        images_names: list = None,
        models_selection: str = BEST_MODELS,
        ms: int = BLOCK_MS_NORMAL
):
    mt.get_function_name(ack=True, tabs=0)
    models, grid, im_size = __od_get_models(models_selection)

    if cv_imgs_orig is None:
        images_names = [mtt.DOGS1, mtt.PERSON, mtt.FACES]
        cv_imgs_orig = [load_img_from_web(image_name) for image_name in images_names]
    else:
        if images_names is None:
            images_names = []
            for i in range(len(cv_imgs_orig)):
                images_names.append('unknown{}'.format(i))
        if len(images_names) != len(cv_imgs_orig):
            mt.exception_error('lists length differ {}, {}'.format(len(images_names), len(cv_imgs_orig)),
                               real_exception=False)
            return

    save_dir = '{}/{}/{}/{}'.format(mtt.IMAGES_OUTPUTS, mt.get_function_name(), SAVE_SCOPE, models_selection)
    mt.create_dir(save_dir)
    __od_models_images_test(
        models=models,
        images_names=images_names,
        cv_imgs_orig=cv_imgs_orig,
        grid=grid,
        delay_ms=ms,
        save_dir=save_dir,
        display_im_size=im_size
    )
    return


def od_best_model_video_test(
        m_name: str = OD_BEST_MODEL,
        video_path: str = None,
        video_name: str = 'unknown',
        work: int = 80
):
    mt.get_function_name(ack=True, tabs=0)
    od_models_compare_video_test(
        video_path=video_path,
        video_name=video_name,
        models_selection=m_name,
        work=work
    )
    return


def od_models_compare_video_test(
        video_path: str = None,
        video_name: str = 'unknown',
        work: int = 80,
        models_selection: str = BEST_MODELS
):
    mt.get_function_name(ack=True, tabs=0)
    models, grid, im_size = __od_get_models(models_selection)
    if video_path is None:  # default video
        video_name = mtt.DOG1
        video_path = get_vid_from_web(name=video_name)

    if not os.path.exists(video_path):
        mt.exception_error(mt.NOT_FOUND.format(video_path), real_exception=False)
        return
    cap = cv2.VideoCapture(video_path)
    if cap.isOpened():
        video_total_frames = cvt.get_frames_from_cap(cap)
        print('\tvid {} has {} frames'.format(video_name, video_total_frames))
        save_dir = '{}/{}/{}/{}'.format(mtt.VIDEOS_OUTPUTS, mt.get_function_name(), SAVE_SCOPE, models_selection)
        mt.create_dir(save_dir)

        __od_models_cap_test(
            models=models,
            cap=cap,
            total_frames=video_total_frames,
            work_every_x_frames=work,
            grid=grid,
            delay_ms=1,
            save_dir=save_dir,
            display_im_size=im_size,
            cap_desc=video_name,
        )
    else:
        mt.exception_error('cap is closed.', real_exception=False)
    return


def od_best_model_cam_test(
        m_name: str = OD_BEST_MODEL,
        total_frames: int = MODEL_FRAMES_CAM
):
    mt.get_function_name(ack=True, tabs=0)
    od_models_compare_cam_test(
        models_selection=m_name,
        total_frames=total_frames
    )
    return


def od_models_compare_cam_test(
        models_selection: str = BEST_MODELS,
        total_frames: int = MODEL_FRAMES_CAM
):
    mt.get_function_name(ack=True, tabs=0)
    models, grid, im_size = __od_get_models(models_selection)
    cam = cvt.CameraWu.open_camera(port=0, type_cam='cv2')
    if cam is not None:
        save_dir = '{}/{}/{}/{}'.format(mtt.VIDEOS_OUTPUTS, mt.get_function_name(), SAVE_SCOPE, models_selection)
        mt.create_dir(save_dir)
        __od_models_cap_test(
            models=models,
            cap=cam,
            total_frames=total_frames,
            work_every_x_frames=1,
            grid=grid,
            delay_ms=1,
            save_dir=save_dir,
            display_im_size=im_size,
            cap_desc='cam 0'
        )
    else:
        mt.exception_error('cap is closed.', real_exception=False)
    return


def add_text_test():
    mt.get_function_name(ack=True, tabs=0)
    img = load_img_from_web(mtt.HORSES)
    cvt.add_text(img, header='test text', pos=(100, 100), text_color='r', with_rect=True, bg_color='y', bg_font_scale=2)
    cvt.add_text(img, header='test text', pos=(100, 200), text_color='black', with_rect=True, bg_color='b',
                 bg_font_scale=1)
    cvt.display_open_cv_image(img, ms=BLOCK_MS_NORMAL, loc=pyplt.Location.CENTER_CENTER.value)
    cv2.destroyAllWindows()
    return


def add_header_test():
    mt.get_function_name(ack=True, tabs=0)
    img = load_img_from_web(mtt.HORSES)

    cvt.add_header(img, header='TOP_LEFT', loc=pyplt.Location.TOP_LEFT.value,
                   text_color='lime', with_rect=True, bg_color='azure', bg_font_scale=1)
    cvt.add_header(img, header='BOTTOM_LEFT', loc=pyplt.Location.BOTTOM_LEFT.value,
                   text_color='fuchsia', with_rect=True, bg_color='black', bg_font_scale=2)
    cvt.display_open_cv_image(img, ms=BLOCK_MS_NORMAL, loc=pyplt.Location.CENTER_CENTER.value)
    cv2.destroyAllWindows()

    img = load_img_from_web(mtt.DOG)
    cvt.display_open_cv_image(
        img,
        ms=BLOCK_MS_NORMAL,
        loc=pyplt.Location.CENTER_CENTER.value,
        header='direct header into display_open_cv_image'
    )
    cv2.destroyAllWindows()
    return


def Mp4_creator_test():
    mt.get_function_name(ack=True, tabs=0)
    # now open video file
    vid_name = mtt.DOG1
    video_path = get_vid_from_web(name=vid_name)

    if not os.path.exists(video_path):
        mt.exception_error(mt.NOT_FOUND.format(video_path), real_exception=False)
        return
    cap = cv2.VideoCapture(video_path)
    if cap.isOpened():
        out_dims = cvt.get_dims_from_cap(cap)
        video_total_frames = cvt.get_frames_from_cap(cap)
        print('\tvid {} has {} frames'.format(vid_name, video_total_frames))
        print('\tvid size is {}'.format(out_dims))
    else:
        mt.exception_error('cap is closed.', real_exception=False)
        return

    out_dir = '{}/create_mp4_test'.format(mtt.VIDEOS_OUTPUTS)
    mt.create_dir(out_dir)
    out_fp = '{}/{}_output.mp4'.format(out_dir, vid_name)

    mp4_creator = cvt.Mp4_creator(
        out_full_path=out_fp,
        out_fps=20.0,
        out_dims=out_dims
    )
    print(mp4_creator)

    for i in range(video_total_frames):
        success, frame = cap.read()
        if i % int(video_total_frames / 10) != 0:  # s
            # do only 10 frames
            continue
        print('\tframe {}/{}:'.format(i + 1, video_total_frames))
        # print('\t\t{}'.format(mt.to_str(frame)))
        if success:
            cvt.add_header(
                frame,
                header='create_mp4_test frame {}/{}'.format(i + 1, video_total_frames),
                loc=pyplt.Location.BOTTOM_LEFT.value,
                text_color=pyplt.get_random_color(),
                bg_color=pyplt.get_random_color(),
            )
            cvt.display_open_cv_image(frame, ms=1, title=vid_name, loc=None,
                                      header='{}/{}'.format(i + 1, video_total_frames))
            mp4_creator.add_frame(frame, ack=True, tabs=2)

    cap.release()
    mp4_creator.finalize()
    cv2.destroyAllWindows()
    return


def __pose_models_images_test(
        models: list,
        images_names: list,
        cv_imgs_orig: list,
        grid: tuple,
        delay_ms: int,
        save_dir: (str, None),
        display_im_size: tuple,
):
    """
    AUX FUNCTION
    :param models:
    :param images_names: for debugging (name of classification, save output name ...)
    :param cv_imgs_orig: will classify on this images
    :param grid:
    :param delay_ms:
    :param save_dir:
    :param display_im_size:
    :return:
    """
    # Prepare cv images list
    fps_list = [mt.FPS(summary_title='detection {}'.format(models[i].name)) for i in range(len(models))]
    for cv_img, img_name in zip(cv_imgs_orig, images_names):
        cv_img_per_model = []
        for j, model in enumerate(models):
            cv_img_clone = cv_img.copy()
            fps_list[j].start()
            detections = model.detect_cv_img(
                cv_img=cv_img,
                fp=2,
                ack=False,  # todo False
                tabs=2,
                img_title=img_name
            )
            fps_list[j].update(ack_progress=True, tabs=1, with_title=True)

            model.draw_detections(
                detections,
                colors_d={
                    'joint_c': 'red',
                    'edge_c': 'lightgreen',
                    'text_c': 'white',
                },
                cv_img=cv_img_clone,
                draw_pose=True,
                draw_points=False,  # todo False
                ack=False,  # todo False
                tabs=1,
                img_title=img_name,
            )
            title = 'detections({}) on image {} {}:'.format(len(detections), img_name, cv_img.shape)
            cvt.add_header(cv_img_clone, header=title, loc=pyplt.Location.TOP_LEFT.value, bg_font_scale=2)
            cvt.add_header(cv_img_clone, header=model.name, loc=pyplt.Location.BOTTOM_LEFT.value, bg_font_scale=1)
            cv_img_clone = cv2.resize(cv_img_clone, display_im_size, interpolation=cv2.INTER_AREA)
            cv_img_per_model.append(cv_img_clone)

        if save_dir is not None:
            save_path = '{}/{}.jpg'.format(save_dir, img_name)
        else:
            save_path = None

        cvt.display_open_cv_images(
            cv_img_per_model,
            ms=delay_ms,
            title='{}'.format(img_name),
            loc=pyplt.Location.CENTER_CENTER.value,
            resize=None,
            grid=grid,
            header=None,
            save_path=save_path
        )
        cv2.destroyAllWindows()
    for fps in fps_list:
        fps.finalize()
    return


def pose_best_model_images_test(m_name: str = POSE_BEST_MODEL, ms: int = BLOCK_MS_NORMAL):
    mt.get_function_name(ack=True, tabs=0)

    pose_models_compare_images_test(
        models_selection=m_name,
        ms=ms
    )
    return


def pose_models_compare_images_test(
        cv_imgs_orig: list = None,
        images_names: list = None,
        models_selection: str = BEST_MODELS,
        ms: int = BLOCK_MS_NORMAL
):
    mt.get_function_name(ack=True, tabs=0)
    models, grid, im_size = __pose_get_models(models_selection)

    if cv_imgs_orig is None:
        images_names = [mtt.PERSON]
        cv_imgs_orig = [load_img_from_web(image_name) for image_name in images_names]
    else:
        if images_names is None:
            images_names = []
            for i in range(len(cv_imgs_orig)):
                images_names.append('unknown{}'.format(i))
        if len(images_names) != len(cv_imgs_orig):
            mt.exception_error('lists length differ {}, {}'.format(len(images_names), len(cv_imgs_orig)),
                               real_exception=False)
            return

    # images_dir = '{}/Input2'.format(mtt.IMAGES_PATH)
    # images_paths = mt.find_files_in_folder(dir_path=images_dir, file_suffix='PNG', ack=True)
    # images_paths += mt.find_files_in_folder(dir_path=images_dir, file_suffix='jpg', ack=True)
    # images_names = [os.path.basename(img_path).split('.')[0] for img_path in images_paths]
    # cv_imgs_orig = [cvt.load_img(img_path) for img_path in images_paths]

    save_dir = '{}/{}/{}/{}'.format(mtt.IMAGES_OUTPUTS, mt.get_function_name(), SAVE_SCOPE, models_selection)
    if not os.path.exists(save_dir):
        mt.create_dir(save_dir)
    __pose_models_images_test(
        models=models,
        images_names=images_names,
        cv_imgs_orig=cv_imgs_orig,
        grid=grid,
        delay_ms=ms,
        save_dir=save_dir,
        display_im_size=im_size
    )
    return


def pose_best_model_cam_test(
        m_name: str = POSE_BEST_MODEL,
        total_frames: int = MODEL_FRAMES_CAM
):
    mt.get_function_name(ack=True, tabs=0)
    pose_models_compare_cam_test(
        models_selection=m_name,
        total_frames=total_frames
    )
    return


def pose_models_compare_cam_test(
        models_selection: str = BEST_MODELS,
        total_frames: int = MODEL_FRAMES_CAM
):
    mt.get_function_name(ack=True, tabs=0)
    models, grid, im_size = __pose_get_models(models_selection)
    cam = cvt.CameraWu.open_camera(port=0, type_cam='cv2')
    if cam is not None:
        save_dir = '{}/{}/{}/{}'.format(mtt.VIDEOS_OUTPUTS, mt.get_function_name(), SAVE_SCOPE, models_selection)
        mt.create_dir(save_dir)
        __pose_models_cap_test(
            models=models,
            cap=cam,
            total_frames=total_frames,
            work_every_x_frames=1,
            grid=grid,
            delay_ms=1,
            save_dir=save_dir,
            display_im_size=im_size,
            cap_desc='cam 0'
        )
    else:
        mt.exception_error('cap is closed.', real_exception=False)
    return


def __pose_models_cap_test(
        models: list,
        cap: (cv2.VideoCapture, cvt.CameraWu),
        total_frames: int,
        work_every_x_frames: int,
        grid: tuple,
        delay_ms: int,
        save_dir: (str, None),
        display_im_size: tuple,
        cap_desc: str,
):
    """
    AUX FUNCTION
    :param models:
    :param grid:
    :param delay_ms:
    :param display_im_size:
    :return:
    """
    if save_dir is not None:
        out_fp = '{}/{}_detected.mp4'.format(save_dir, cap_desc)
        out_dims = (display_im_size[0] * grid[1], display_im_size[1] * grid[0])
        mp4 = cvt.Mp4_creator(
            out_full_path=out_fp,
            out_fps=20.0,
            out_dims=out_dims
        )
        print(mp4)
        print('\tvid size output will be {}'.format(out_dims))
    else:
        mp4 = None
    fps_list = [mt.FPS(summary_title='detection {}'.format(models[i].name)) for i in range(len(models))]

    for i in range(total_frames):
        if isinstance(cap, cv2.VideoCapture):
            success, cv_img = cap.read()
        else:
            success, cv_img = cap.read_img()
        if i % work_every_x_frames != 0:  # s
            # do only 10 frames
            continue
        print('\tframe {}/{}:'.format(i + 1, total_frames))
        if success:
            cv_img_per_model = []
            for j, model in enumerate(models):
                cv_img_clone = cv_img.copy()
                fps_list[j].start()
                detections = model.detect_cv_img(
                    cv_img=cv_img,
                    fp=2,
                    ack=False,  # todo False
                    tabs=1,
                    img_title='frame {}'.format(i + 1)
                )
                fps_list[j].update(ack_progress=True, tabs=2, with_title=True)

                model.draw_detections(
                    detections,
                    colors_d={
                        'joint_c': 'red',
                        'edge_c': 'lightgreen',
                        'text_c': 'white',
                    },
                    cv_img=cv_img_clone,
                    draw_pose=True,
                    draw_points=False,
                    ack=False,  # todo False
                    tabs=1,
                    img_title='frame {}'.format(i + 1),
                )
                title = 'detections({}) on frame {}/{} {}:'.format(len(detections), (i + 1), total_frames, cv_img.shape)
                cvt.add_header(cv_img_clone, header=title, loc=pyplt.Location.TOP_LEFT.value, bg_font_scale=2)
                cvt.add_header(cv_img_clone, header=model.name, loc=pyplt.Location.BOTTOM_LEFT.value, bg_font_scale=1)
                cv_img_clone = cv2.resize(cv_img_clone, display_im_size, interpolation=cv2.INTER_AREA)
                cv_img_per_model.append(cv_img_clone)
            if mp4 is not None:
                big_img = cvt.unpack_list_imgs_to_big_image(cv_img_per_model, grid=grid, resize=None)
                mp4.add_frame(big_img)
            cv_title = '{} models'.format(len(models)) if len(models) > 1 else models[0].name
            cv_title += ' on {}'.format(cap_desc)
            cvt.display_open_cv_images(
                cv_img_per_model,
                ms=delay_ms,
                title=cv_title,
                loc=pyplt.Location.CENTER_CENTER.value,
                resize=None,
                grid=grid,
                header=None,
                save_path=None
            )
    for fps in fps_list:
        fps.finalize()
    cv2.destroyAllWindows()
    if mp4 is not None:
        mp4.finalize()
    return


def test_all():
    print('{}{}:'.format('-' * 5, mt.get_base_file_and_function_name()))
    get_cv_version_test()
    imread_imwrite_test()
    list_to_cv_image_test()
    display_open_cv_image_test()
    display_open_cv_image_loop_test()
    resize_opencv_image_test()
    move_cv_img_x_y_test()
    move_cv_img_by_str_test()
    unpack_list_imgs_to_big_image_test()
    display_open_cv_images_test()
    display_open_cv_images_loop_test()
    gray_to_BGR_and_back_test()
    BGR_img_to_RGB_and_back_test()
    add_header_test()
    add_text_test()
    CameraWu_cv2_test()
    CameraWu_acapture_test()
    CameraWu_imutils_test()
    od_best_model_images_test()
    od_best_model_video_test()
    od_best_model_cam_test()
    od_models_compare_images_test()
    od_models_compare_video_test()
    od_models_compare_cam_test()
    Mp4_creator_test()
    pose_best_model_images_test()
    pose_best_model_cam_test()
    print('{}'.format('-' * 20))
    return



from enum import Enum
import wizzi_utils.misc as mt
from wizzi_utils.open_cv.labels_bank import PASCAL_VOC_2012_21_LABELS
from wizzi_utils.open_cv.labels_bank import PASCAL_VOC_2012_20_LABELS
from wizzi_utils.open_cv.labels_bank import COCO_YOLO_80_LABELS
from wizzi_utils.open_cv.labels_bank import ILSVRC2016_201_LABELS
from wizzi_utils.open_cv.labels_bank import COCO_183_LABELS
from wizzi_utils.open_cv.labels_bank import COCO_182_LABELS


# from wizzi_utils_test.downloaded_models.labels_bank import IMAGENET


class Jobs(Enum):
    OBJECT_DETECTION = 'object_detection'
    SEGMENTATION = 'segmentation'
    OPEN_POSE = 'openpose'
    CLASSIFICATION = 'classification'


class DnnFamily(Enum):
    Caffe = 'Caffe'
    Darknet = 'Darknet'
    TF = 'TensorFlow'


class DownloadStyle(Enum):
    Direct = 'Direct'
    Tar = 'tar'
    Zip = 'zip'


test_fps_od_image_url = 'https://raw.githubusercontent.com/pjreddie/darknet/master/data/dog.jpg'
test_fps_od_iters = 10
test_fps_computer_info = 'AMD64, Intel64 Family 6 Model 158 Stepping 9, GenuineIntel, ' + \
                         'Physical cores 4, Total cores 8, Frequency 3601.00Mhz, CPU Usage 13.6%)'

fps_info = '({} iterations, the above cfg, on image {} on {})'.format(test_fps_od_iters, test_fps_od_image_url,
                                                                      test_fps_computer_info)
# see wu.cvt.test.fps_models_test()

# models from: https://github.com/opencv/opencv_extra/tree/master/testdata/dnn
MODELS_DNN_OBJECT_DETECTION_META_DATA = {
    # see wu.cvt.get_all_object_detection_models_info(ack=True)
    'yolov4': {  # checked
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Darknet.value,
        # YOLO FAMILY ALLOWED IN_DIMS: (320, 320), (416, 416), (512, 512), (608, 608)
        'in_dims': (608, 608),
        # 'scalefactor': 1 / 255,
        'scalefactor': 1 / 127.5,
        'mean': (0, 0, 0),
        'swapRB': True,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': COCO_YOLO_80_LABELS,
        'URL': {
            'cfg': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/yolov4.cfg',
            'weights': 'https://github.com/AlexeyAB/darknet/releases/download/darknet_yolo_v3_optimal/yolov4.weights',
            'cfg_download_style': DownloadStyle.Direct.value,
            'weights_download_style': DownloadStyle.Direct.value,
        },
        'info': {
            'web_name': 'yolov4.weights',
            'size': 'weights({}), cfg({})'.format(mt.add_color('245.78 MB', ops='c'),
                                                  mt.add_color('11.94 KB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('1.44 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'yolov3': {  # checked
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Darknet.value,
        # YOLO FAMILY ALLOWED IN_DIMS: (320, 320), (416, 416), (512, 512), (608, 608)
        'in_dims': (416, 416),
        'scalefactor': 1 / 255,
        'mean': (0, 0, 0),
        'swapRB': True,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': COCO_YOLO_80_LABELS,
        'URL': {
            'cfg': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/yolov3.cfg',
            'weights': 'https://pjreddie.com/media/files/yolov3.weights',
            'cfg_download_style': DownloadStyle.Direct.value,
            'weights_download_style': DownloadStyle.Direct.value,
        },
        'info': {
            'web_name': 'yolov3.weights',
            'size': 'weights({}), cfg({})'.format(mt.add_color('236.52 MB', ops='c'), mt.add_color('8.22 KB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('3.24 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'yolov3-ssp': {  # checked
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Darknet.value,
        # YOLO FAMILY ALLOWED IN_DIMS: (320, 320), (416, 416), (512, 512), (608, 608)
        'in_dims': (608, 608),
        'scalefactor': 1 / 127.5,
        'mean': (0, 0, 0),
        'swapRB': True,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': COCO_YOLO_80_LABELS,
        'URL': {
            'cfg': 'https://raw.githubusercontent.com/pjreddie/darknet/master/cfg/yolov3-spp.cfg',
            'weights': 'https://github.com/ultralytics/yolov3/releases/download/v8/yolov3-spp.weights',
            'cfg_download_style': DownloadStyle.Direct.value,
            'weights_download_style': DownloadStyle.Direct.value,
        },
        'info': {
            'web_name': 'yolov3-spp.weights',
            'size': 'weights({}), cfg({})'.format(mt.add_color('240.53 MB', ops='c'), mt.add_color('8.4 KB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('1.67 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'yolo-voc': {  # checked
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Darknet.value,
        # YOLO FAMILY ALLOWED IN_DIMS: (320, 320), (416, 416), (512, 512), (608, 608)
        'in_dims': (416, 416),
        'scalefactor': 1 / 255,
        'mean': (127.5, 127.5, 127.5),
        'swapRB': True,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': PASCAL_VOC_2012_20_LABELS,
        'URL': {
            'cfg': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/yolo-voc.cfg',
            'weights': 'https://pjreddie.com/media/files/yolo-voc.weights',
            'cfg_download_style': DownloadStyle.Direct.value,
            'weights_download_style': DownloadStyle.Direct.value,
        },
        'info': {
            'web_name': 'yolo-voc.weights',
            'size': 'weights({}), cfg({})'.format(mt.add_color('193.31 MB', ops='c'), mt.add_color('2.66 KB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('7.48 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'MobileNetSSD_deploy': {  # checked
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Caffe.value,
        'in_dims': (800, 600),
        'scalefactor': 1 / 127.5,
        'mean': (127.5, 127.5, 127.5),
        'swapRB': False,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': PASCAL_VOC_2012_21_LABELS,
        'URL': {
            'prototxt': 'https://raw.githubusercontent.com/chuanqi305/MobileNet-SSD/' +
                        'daef68a6c2f5fbb8c88404266aa28180646d17e0/MobileNetSSD_deploy.prototxt',
            'caffemodel': 'https://drive.google.com/uc?export=download&id=0B3gersZ2cHIxRm5PMWRoTkdHdHc',
            'prototxt_download_style': DownloadStyle.Direct.value,
            'caffemodel_download_style': DownloadStyle.Direct.value,
        },
        'info': {
            'web_name': 'MobileNetSSD_deploy',
            'size': 'caffemodel({}), prototxt({})'.format(mt.add_color('22.08 MB', ops='c'),
                                                          mt.add_color('28.67 KB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('13.28 FPS'), fps_info),
            'info': 'https://www.programmersought.com/article/569030384/',
        },
    },
    'yolov4-tiny': {  # checked
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Darknet.value,
        # YOLO FAMILY ALLOWED IN_DIMS: (320, 320), (416, 416), (512, 512), (608, 608)
        'in_dims': (608, 608),
        'scalefactor': 1 / 127.5,
        'mean': (0, 0, 0),
        'swapRB': True,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': COCO_YOLO_80_LABELS,
        'URL': {
            'cfg': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/yolov4-tiny.cfg',
            'weights': 'https://github.com/AlexeyAB/darknet/releases/download/darknet_yolo_v4_pre/yolov4-tiny.weights',
            'cfg_download_style': DownloadStyle.Direct.value,
            'weights_download_style': DownloadStyle.Direct.value,
        },
        'info': {
            'web_name': 'yolov4-tiny.weights',
            'size': 'weights({}), cfg({})'.format(mt.add_color('23.13 MB', ops='c'), mt.add_color('2.96 KB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('11.33 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'yolov3_tiny': {  # checked
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Darknet.value,
        # YOLO FAMILY ALLOWED IN_DIMS: (320, 320), (416, 416), (512, 512), (608, 608)
        'in_dims': (512, 512),
        'scalefactor': 1 / 127.5,
        'mean': (0, 0, 0),
        'swapRB': True,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': COCO_YOLO_80_LABELS,
        'URL': {
            'cfg': 'https://raw.githubusercontent.com/pjreddie/darknet/master/cfg/yolov3-tiny.cfg',
            'weights': 'https://github.com/ultralytics/yolov3/releases/download/v8/yolov3-tiny.weights',
            'cfg_download_style': DownloadStyle.Direct.value,
            'weights_download_style': DownloadStyle.Direct.value,
        },
        'info': {
            'web_name': 'yolov3-tiny.weights',
            'size': 'weights({}), cfg({})'.format(mt.add_color('33.79 MB', ops='c'), mt.add_color('1.87 KB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('18.56 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'yolov2-tiny-voc': {  # checked
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Darknet.value,
        # YOLO FAMILY ALLOWED IN_DIMS: (320, 320), (416, 416), (512, 512), (608, 608)
        'in_dims': (416, 416),
        'scalefactor': 1 / 127.5,
        'mean': (0, 0, 0),
        'swapRB': True,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': PASCAL_VOC_2012_20_LABELS,
        'URL': {
            'cfg': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/tiny-yolo-voc.cfg',
            'weights': 'https://pjreddie.com/media/files/yolov2-tiny-voc.weights',
            'cfg_download_style': DownloadStyle.Direct.value,
            'weights_download_style': DownloadStyle.Direct.value,
        },
        'info': {
            'web_name': 'yolov2-tiny-voc.weights',
            'size': 'weights({}), cfg({})'.format(mt.add_color('60.53 MB', ops='c'), mt.add_color('1.38 KB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('24.61 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'VGG_ILSVRC2016_SSD_300x300_deploy': {  # checked
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Caffe.value,
        'in_dims': (300, 300),
        'scalefactor': 1,
        'mean': (127, 127, 127),
        'swapRB': False,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': ILSVRC2016_201_LABELS,
        'URL': {
            'prototxt': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/ssd_vgg16.prototxt',
            'caffemodel': 'https://www.dropbox.com/s/8apyk3uzk2vl522/' +
                          'VGG_ILSVRC2016_SSD_300x300_iter_440000.caffemodel?dl=1',
            'prototxt_download_style': DownloadStyle.Direct.value,
            'caffemodel_download_style': DownloadStyle.Direct.value,
        },
        'info': {
            'web_name': 'VGG_ILSVRC2016_SSD_300x300_iter_440000.caffemodel',
            'size': 'caffemodel({}), prototxt({})'.format(mt.add_color('192.06 MB', ops='c'),
                                                          mt.add_color('23.93 KB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('2.58'), fps_info),
            'info': 'TODO',
        },
    },
    'VGG_ILSVRC_16_layers': {  # checked
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Caffe.value,
        'in_dims': (800, 600),
        'scalefactor': 1,
        'mean': (102.9801, 115.9465, 122.7717),
        'swapRB': False,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': PASCAL_VOC_2012_21_LABELS,
        'need_normalize': 'normalize output needed',
        'URL': {
            'prototxt': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/' +
                        'faster_rcnn_vgg16.prototxt',
            'caffemodel': 'https://dl.dropboxusercontent.com/s/o6ii098bu51d139/faster_rcnn_models.tgz?dl=0',
            'prototxt_download_style': DownloadStyle.Direct.value,
            'caffemodel_download_style': DownloadStyle.Tar.value,
            'caffemodel_name': 'VGG16_faster_rcnn_final.caffemodel',
        },
        'info': {
            'web_name': 'VGG16_faster_rcnn_final.caffemodel',
            'size': 'caffemodel({}), prototxt({})'.format(mt.add_color('522.92 MB', ops='c'),
                                                          mt.add_color('8.45 KB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('0.21 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'rfcn_pascal_voc_resnet50': {  # checked
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Caffe.value,
        'in_dims': (800, 600),
        'scalefactor': 1,
        'mean': (102.9801, 115.9465, 122.7717),
        'swapRB': False,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': PASCAL_VOC_2012_21_LABELS,
        'need_normalize': 'normalize output needed',
        'URL': {
            'prototxt': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/' +
                        'rfcn_pascal_voc_resnet50.prototxt',
            'caffemodel': 'https://onedrive.live.com/download?' +
                          'cid=10B28C0E28BF7B83&resid=10B28C0E28BF7B83%215317&authkey=%21AIeljruhoLuail8',
            'prototxt_download_style': DownloadStyle.Direct.value,
            'caffemodel_download_style': DownloadStyle.Tar.value,
            'caffemodel_name': 'resnet50_rfcn_final.caffemodel',
        },
        'info': {
            'web_name': 'resnet50_rfcn_final.caffemodel',
            'size': 'caffemodel({}), prototxt({})'.format(mt.add_color('121.58 MB', ops='c'),
                                                          mt.add_color('62.8 KB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('1.28 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'faster_rcnn_zf': {  # checked
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Caffe.value,
        'in_dims': (800, 600),
        'scalefactor': 1,
        'mean': (102.9801, 115.9465, 122.7717),
        'swapRB': False,
        'crop': False,
        'threshold': 0.4,
        'nms_threshold': 0.0,
        'labels_dict': PASCAL_VOC_2012_21_LABELS,
        'need_normalize': 'normalize output needed',
        'URL': {
            'prototxt': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/' +
                        'faster_rcnn_zf.prototxt',
            'caffemodel': 'https://dl.dropboxusercontent.com/s/o6ii098bu51d139/' +
                          'faster_rcnn_models.tgz?dl=0',
            'prototxt_download_style': DownloadStyle.Direct.value,
            'caffemodel_download_style': DownloadStyle.Tar.value,
            'caffemodel_name': 'ZF_faster_rcnn_final.caffemodel',
        },
        'info': {
            'web_name': 'ZF_faster_rcnn_final',
            'size': 'caffemodel({}), prototxt({})'.format(mt.add_color('226.19 MB', ops='c'),
                                                          mt.add_color('7.1 KB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('0.84 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'ssd_inception_v2_coco_2017_11_17': {  # checked
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.TF.value,
        'in_dims': (800, 600),
        'scalefactor': 1,
        'mean': (0, 0, 0),
        'swapRB': True,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': COCO_183_LABELS,
        'URL': {
            'pbtxt': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/' +
                     'ssd_inception_v2_coco_2017_11_17.pbtxt',
            'pb': 'http://download.tensorflow.org/models/object_detection/ssd_inception_v2_coco_2017_11_17.tar.gz',
            'pbtxt_download_style': DownloadStyle.Direct.value,
            'pb_download_style': DownloadStyle.Tar.value,
            'pb_name': 'frozen_inference_graph.pb',
        },
        'info': {
            'web_name': 'ssd_inception_v2_coco_2017_11_17',
            'size': 'pb({}), pbtxt({})'.format(mt.add_color('97.26 MB', ops='c'), mt.add_color('114.77 KB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('3.78 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'ssd_mobilenet_v1_coco': {  # checked
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.TF.value,
        'in_dims': (416, 416),
        'scalefactor': 1 / 127.5,
        'mean': (0, 0, 0),
        'swapRB': True,
        'crop': False,
        'threshold': 0.4,
        'nms_threshold': 0.4,
        'labels_dict': COCO_183_LABELS,
        'URL': {
            'pbtxt': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/' +
                     'ssd_mobilenet_v1_coco.pbtxt',
            'pb': 'http://download.tensorflow.org/models/object_detection/ssd_mobilenet_v1_coco_11_06_2017.tar.gz',
            'pbtxt_download_style': DownloadStyle.Direct.value,
            'pb_download_style': DownloadStyle.Tar.value,
            'pb_name': 'frozen_inference_graph.pb',
        },
        'info': {
            'web_name': 'ssd_mobilenet_v1_coco_11_06_2017',
            'size': 'pb({}), pbtxt({})'.format(mt.add_color('27.76 MB', ops='c'), mt.add_color('62.08 KB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('27.13 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'faster_rcnn_inception_v2_coco_2018_01_28': {  # checked
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.TF.value,
        'in_dims': (800, 600),
        'scalefactor': 1,
        'mean': (0, 0, 0),
        'swapRB': True,
        'crop': False,
        'threshold': 0.4,
        'nms_threshold': 0.4,
        'labels_dict': COCO_182_LABELS,
        'URL': {
            'pbtxt': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/' +
                     'faster_rcnn_inception_v2_coco_2018_01_28.pbtxt',
            'pb': 'http://download.tensorflow.org/models/object_detection/' +
                  'faster_rcnn_inception_v2_coco_2018_01_28.tar.gz',
            'pbtxt_download_style': DownloadStyle.Direct.value,
            'pb_download_style': DownloadStyle.Tar.value,
            'pb_name': 'frozen_inference_graph.pb',
        },
        'info': {
            'web_name': 'faster_rcnn_inception_v2_coco_2018_01_28',
            'size': 'pb({}), pbtxt({})'.format(mt.add_color('54.51 MB', ops='c'), mt.add_color('112.92 KB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('2.51 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'faster_rcnn_resnet50_coco_2018_01_28': {  # checked
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.TF.value,
        'in_dims': (800, 600),
        'scalefactor': 1,
        'mean': (0, 0, 0),
        'swapRB': True,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': COCO_182_LABELS,
        # 'need_normalize': '??? normalize output needed',
        'URL': {
            'pbtxt': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/' +
                     'faster_rcnn_resnet50_coco_2018_01_28.pbtxt',
            'pb': 'http://download.tensorflow.org/models/object_detection/faster_rcnn_resnet50_coco_2018_01_28.tar.gz',
            'pbtxt_download_style': DownloadStyle.Direct.value,
            'pb_download_style': DownloadStyle.Tar.value,
            'pb_name': 'frozen_inference_graph.pb',
        },
        'info': {
            'web_name': 'faster_rcnn_resnet50_coco_2018_01_28',
            'size': 'pb({}), pbtxt({})'.format(mt.add_color('114.97 MB', ops='c'), mt.add_color('88.76 KB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('0.84 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'ssd_mobilenet_v1_coco_2017_11_17': {  # checked
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.TF.value,
        'in_dims': (300, 300),
        'scalefactor': 1,
        'mean': (0, 0, 0),
        'swapRB': True,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': COCO_183_LABELS,
        'URL': {
            'pbtxt': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/' +
                     'ssd_mobilenet_v1_coco_2017_11_17.pbtxt',
            'pb': 'http://download.tensorflow.org/models/object_detection/ssd_mobilenet_v1_coco_2017_11_17.tar.gz',
            'pbtxt_download_style': DownloadStyle.Direct.value,
            'pb_download_style': DownloadStyle.Tar.value,
            'pb_name': 'frozen_inference_graph.pb',
        },
        'info': {
            'web_name': 'ssd_mobilenet_v1_coco_2017_11_17',
            'size': 'pb({}), pbtxt({})'.format(mt.add_color('27.76 MB', ops='c'), mt.add_color('62.08 KB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('46.28 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'ssd_mobilenet_v1_ppn_coco': {  # checked
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.TF.value,
        'in_dims': (300, 300),
        'scalefactor': 1,
        'mean': (127.5, 127.5, 127.5),
        'swapRB': False,
        'crop': False,
        'threshold': 0.4,
        'nms_threshold': 0.4,
        'labels_dict': COCO_183_LABELS,
        'URL': {
            'pbtxt': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/' +
                     'ssd_mobilenet_v1_ppn_coco.pbtxt',
            'pb': 'http://download.tensorflow.org/models/object_detection/' +
                  'ssd_mobilenet_v1_ppn_shared_box_predictor_300x300_coco14_sync_2018_07_03.tar.gz',
            'pbtxt_download_style': DownloadStyle.Direct.value,
            'pb_download_style': DownloadStyle.Tar.value,
            'pb_name': 'frozen_inference_graph.pb',
        },
        'info': {
            'web_name': 'ssd_mobilenet_v1_ppn_shared_box_predictor_300x300_coco14_sync_2018_07_03',
            'size': 'pb({}), pbtxt({})'.format(mt.add_color('10.29 MB', ops='c'), mt.add_color('67.4 KB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('38.72 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'ssd_mobilenet_v2_coco_2018_03_29': {
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.TF.value,
        'in_dims': (800, 600),
        'scalefactor': 1,
        'mean': (0, 0, 0),
        'swapRB': True,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': COCO_183_LABELS,
        'URL': {
            'pbtxt': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/' +
                     'ssd_mobilenet_v2_coco_2018_03_29.pbtxt',
            'pb': 'http://download.tensorflow.org/models/object_detection/ssd_mobilenet_v2_coco_2018_03_29.tar.gz',
            'pbtxt_download_style': DownloadStyle.Direct.value,
            'pb_download_style': DownloadStyle.Tar.value,
            'pb_name': 'frozen_inference_graph.pb',
        },
        'info': {
            'web_name': 'ssd_mobilenet_v2_coco_2018_03_29',
            'size': 'pb({}), pbtxt({})'.format(mt.add_color('66.46 MB', ops='c'), mt.add_color('112.89 KB', ops='c')),
            'fps': '{} {}'.format(mt.add_color('6.57 FPS'), fps_info),
            'info': 'TODO',
        },
    },
    'opencv_face_detector': {
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Caffe.value,
        'in_dims': (520, 520),
        'scalefactor': 1,
        'mean': (104, 177, 123),
        'swapRB': False,
        'crop': False,
        'threshold': 0.5,
        'nms_threshold': None,
        'labels_dict': {
            'desc': 'TODO',
            'num_classes': 1,
            'labels': ['None', 'face']
        },
        'URL': {
            'prototxt': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/' +
                        'opencv_face_detector.prototxt',
            'caffemodel': 'https://github.com/opencv/opencv_3rdparty/raw/dnn_samples_face_detector_20170830/' +
                          'res10_300x300_ssd_iter_140000.caffemodel',
            'prototxt_download_style': DownloadStyle.Direct.value,
            'caffemodel_download_style': DownloadStyle.Direct.value,
            'info1': 'https://www.programmersought.com/article/16544476883/',
            'info2': 'https://www.pyimagesearch.com/2018/02/26/face-detection-with-opencv-and-deep-learning/',
        },
        'info': {
            'web_name': 'res10_300x300_ssd_iter_140000',
            'size': 'caffemodel({}), prototxt({})'.format(mt.add_color('10.17 MB', ops='c'),
                                                          mt.add_color('27.45 KB', ops='c')),
            'fps': 'TODO {}'.format(fps_info),
            'info': 'https://www.programmersought.com/article/16544476883/',
        },
    },
}
MODELS_DNN_SEGMENTATION_META_DATA = {
    'fcn8s-heavy-pascal': {
        'job': Jobs.SEGMENTATION.value,
        'family': DnnFamily.Caffe.value,
        'in_dims': 'x',
        'scalefactor': 'x',
        'mean': 'x',
        'swapRB': False,
        'crop': False,
        'threshold': 0,
        'nms_threshold': 0,
        'labels_dict': 'x',
        'need_normalize': '??? normalize output needed',
        'URL': {
            'prototxt': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/' +
                        'fcn8s-heavy-pascal.prototxt',
            'caffemodel': 'http://dl.caffe.berkeleyvision.org/fcn8s-heavy-pascal.caffemodel',
            'prototxt_download_style': DownloadStyle.Direct.value,
            'caffemodel_download_style': DownloadStyle.Direct.value,
        },
        'info': {
            'web_name': 'TODO',
            'size': 'TODO',
            'fps': 'TODO {}'.format(fps_info),
            'info': 'TODO',
        },
    },
}
MODELS_DNN_CLASSIFICATION_META_DATA = {

}
MODELS_OPEN_POSE_META_DATA = {
    'openpose_pose_mpi': {
        'job': Jobs.OPEN_POSE.value,
        'family': DnnFamily.Caffe.value,
        'num_points': 15,
        'points_name': {  # not used for now
            "Head": 0, "Neck": 1, "RShoulder": 2, "RElbow": 3, "RWrist": 4,
            "LShoulder": 5, "LElbow": 6, "LWrist": 7, "RHip": 8, "RKnee": 9,
            "RAnkle": 10, "LHip": 11, "LKnee": 12, "LAnkle": 13, "Chest": 14,
            "Background": 15
        },
        'point_pairs': [
            [0, 1], [1, 2], [2, 3], [3, 4], [1, 5],
            [5, 6], [6, 7], [1, 14], [14, 8], [8, 9],
            [9, 10], [14, 11], [11, 12], [12, 13]
        ],
        'in_dims': (368, 368),
        'scalefactor': 1 / 255,
        'mean': (0, 0, 0),
        'swapRB': False,
        'crop': False,
        'threshold': 0.1,
        'URL': {
            'prototxt': 'https://raw.githubusercontent.com/CMU-Perceptual-Computing-Lab/openpose/master/models/' +
                        'pose/mpi/pose_deploy_linevec_faster_4_stages.prototxt',
            'caffemodel': 'http://posefs1.perception.cs.cmu.edu/OpenPose/models/pose/mpi/pose_iter_160000.caffemodel',
            'prototxt_download_style': DownloadStyle.Direct.value,
            'caffemodel_download_style': DownloadStyle.Direct.value,
        },
        'info': {
            'web_name': 'pose_iter_160000',
            'size': 'caffemodel({}), prototxt({})'.format(mt.add_color('196.41 MB', ops='c'),
                                                          mt.add_color('45.83 KB', ops='c')),
            'fps': 'TODO {}'.format(fps_info),
            'info': 'https://www.programmersought.com/article/3282857837/',
        },
    },
    'openpose_pose_coco': {
        'job': Jobs.OPEN_POSE.value,
        'family': DnnFamily.Caffe.value,
        'num_points': 18,
        'points_name': {  # not used for now
            "Nose": 0, "Neck": 1, "RShoulder": 2, "RElbow": 3, "RWrist": 4,
            "LShoulder": 5, "LElbow": 6, "LWrist": 7, "RHip": 8, "RKnee": 9,
            "RAnkle": 10, "LHip": 11, "LKnee": 12, "LAnkle": 13, "REye": 14,
            "LEye": 15, "REar": 16, "LEar": 17, "Background": 18
        },
        'point_pairs': [
            [1, 0], [1, 2], [1, 5], [2, 3], [3, 4], [5, 6],
            [6, 7], [1, 8], [8, 9], [9, 10], [1, 11], [11, 12],
            [12, 13], [0, 14], [0, 15], [14, 16], [15, 17]
        ],
        'in_dims': (368, 368),
        'scalefactor': 1 / 255,
        'mean': (0, 0, 0),
        'swapRB': False,
        'crop': False,
        'threshold': 0.1,
        'URL': {
            'prototxt': 'https://raw.githubusercontent.com/CMU-Perceptual-Computing-Lab/openpose/master/models/pose/' +
                        'coco/pose_deploy_linevec.prototxt',
            'caffemodel': 'http://posefs1.perception.cs.cmu.edu/OpenPose/models/pose/coco/pose_iter_440000.caffemodel',
            'prototxt_download_style': DownloadStyle.Direct.value,
            'caffemodel_download_style': DownloadStyle.Direct.value,
            'info1': 'https://github.com/CMU-Perceptual-Computing-Lab/openpose',
            'info2': 'https://www.programmersought.com/article/3282857837/',
        },
        'info': {
            'web_name': 'pose_iter_440000',
            'size': 'caffemodel({}), prototxt({})'.format(mt.add_color('199.58 MB', ops='c'),
                                                          mt.add_color('45.78 KB', ops='c')),
            'fps': 'TODO {}'.format(fps_info),
            'info': 'TODO',
        },
    },
    'body25': {
        'job': Jobs.OPEN_POSE.value,
        'family': DnnFamily.Caffe.value,
        'num_points': 25,
        'point_pairs': [
            [1, 0], [1, 2], [1, 5], [2, 3], [3, 4], [5, 6],
            [6, 7], [0, 15], [15, 17], [0, 16], [16, 18], [1, 8],
            [8, 9], [9, 10], [10, 11], [11, 22], [22, 23], [11, 24],
            [8, 12], [12, 13], [13, 14], [14, 19], [19, 20], [14, 21]
        ],
        'in_dims': (368, 368),
        'scalefactor': 1 / 255,
        'mean': (0, 0, 0),
        'swapRB': False,
        'crop': False,
        'threshold': 0.1,
        'URL': {
            'prototxt': 'https://raw.githubusercontent.com/CMU-Perceptual-Computing-Lab/' +
                        'openpose/master/models/pose/body_25/pose_deploy.prototxt',
            'caffemodel': 'http://posefs1.perception.cs.cmu.edu/OpenPose/models/pose/body_25/' +
                          'pose_iter_584000.caffemodel',
            'prototxt_download_style': DownloadStyle.Direct.value,
            'caffemodel_download_style': DownloadStyle.Direct.value,
            'info1': 'https://github.com/CMU-Perceptual-Computing-Lab/openpose',
            'info2': 'https://www.programmersought.com/article/3282857837/',
        },
        'info': {
            'web_name': 'pose_iter_584000.caffemodel',
            'size': 'caffemodel({}), prototxt({})'.format(mt.add_color('99.86 MB', ops='c'),
                                                          mt.add_color('41.34 KB', ops='c')),
            'fps': 'TODO {}'.format(fps_info),
            'info': 'TODO',
        },
    },
    'hand_pose': {
        'job': Jobs.OPEN_POSE.value,
        'family': DnnFamily.Caffe.value,
        'num_points': 22,
        'point_pairs': [
            [0, 1], [1, 2], [2, 3], [3, 4],  # thumb
            [0, 5], [5, 6], [6, 7], [7, 8],  # pinkie
            [0, 9], [9, 10], [10, 11], [11, 12],  # middle
            [0, 13], [13, 14], [14, 15], [15, 16],  # ring
            [0, 17], [17, 18], [18, 19], [19, 20]  # small
        ],
        'in_dims': (368, 368),
        'scalefactor': 1 / 255,
        'mean': (0, 0, 0),
        'swapRB': False,
        'crop': False,
        'threshold': 0.1,
        'URL': {
            'prototxt': 'https://raw.githubusercontent.com/CMU-Perceptual-Computing-Lab/openpose/master/models/' +
                        'hand/pose_deploy.prototxt',
            'caffemodel': 'http://posefs1.perception.cs.cmu.edu/OpenPose/models/hand/pose_iter_102000.caffemodel',
            'prototxt_download_style': DownloadStyle.Direct.value,
            'caffemodel_download_style': DownloadStyle.Direct.value,
            'info1': 'https://www.programmersought.com/article/1536716931/',
        },
        'info': {
            'web_name': 'pose_iter_102000.caffemodel',
            'size': 'caffemodel({}), prototxt({})'.format(mt.add_color('140.52 MB', ops='c'),
                                                          mt.add_color('25.83 KB', ops='c')),
            'fps': 'TODO {}'.format(fps_info),
            'info': 'TODO',
        },
    },
}



from wizzi_utils.tflite import tflite_tools as tflt
from wizzi_utils.misc import misc_tools as mt
from wizzi_utils.misc.test import test_misc_tools as mtt
from wizzi_utils.open_cv import open_cv_tools as cvt
from wizzi_utils.open_cv.test import test_open_cv_tools as cvtt
import os
# noinspection PyPackageRequirements
import cv2

SAVE_SCOPE = 'TFL'
OD_BEST_MODEL = 'ssd_mobilenet_v2_mnasfpn'
POSE_BEST_MODEL = 'pose_landmark_heavy'


def get_tflite_version_test():
    mt.get_function_name(ack=True, tabs=0)
    tflt.get_tflite_version(ack=True)
    return


def od_fps_models_test():
    iterations = 10
    images_names = [mtt.DOGS1] * iterations
    cv_imgs_orig = [cvtt.load_img_from_web(image_name) for image_name in images_names]
    print(images_names)
    od_models_compare_images_test(
        cv_imgs_orig=cv_imgs_orig,
        images_names=images_names,
        models_selection=cvtt.ALL_MODELS,
        ms=1
    )
    return


def __od_get_models(models_selection: str = cvtt.BEST_MODELS):
    if models_selection == cvtt.BEST_MODELS:
        # preparing 4 best models
        grid = (2, 2)
        display_im_size = (640, 480)
        models_names = [
            'ssd_mobilenet_v3_small_coco_2020_01_14',
            'ssd_mobilenet_v2_mnasfpn',
            'ssd_mobilenet_v3_large_coco_2020_01_14',
            'ssdlite_mobiledet_cpu_320x320_coco_2020_05_19',
            # 'ssd_mobilenet_v1_1_metadata_1',
            # 'ssdlite_mobilenet_v2_coco_300_integer_quant_with_postprocess',
            # 'coco_ssd_mobilenet_v1_1_0_quant_2018_06_29'
        ]
    elif models_selection == cvtt.ALL_MODELS:
        # preparing all(7) models
        grid = (2, 4)
        display_im_size = (320, 240)
        models_names = tflt.get_all_object_detection_models_info(ack=False)
    else:
        # preparing 1 models - default is BEST_MODEL
        grid = (1, 1)
        display_im_size = (640, 480)
        models_names = [models_selection]
    models = []
    for m_name in models_names:
        save_dir = '{}/{}/{}'.format(mtt.TFL_MODELS, tflt.MODELS_TFL_META_DATA[OD_BEST_MODEL]['job'], m_name)
        m = tflt.TfltObjectDetectionModels(
            save_load_dir=save_dir,
            model_name=m_name,
            # allowed_class=['dog', 'cat'],
            allowed_class=None,
            tabs=1,
            # threshold=0.2,  # take default
            # nms={'score_threshold': 0.4, 'nms_threshold': 0.4},  # take default
        )
        print(m)
        models.append(m)
    return models, grid, display_im_size


def __pose_get_models(models_selection: str = cvtt.BEST_MODELS):
    if models_selection == cvtt.BEST_MODELS:
        # preparing 3 best models
        grid = (1, 3)
        display_im_size = (640, 480)
        models_names = [
            'pose_landmark_lite',
            'pose_landmark_full',
            'pose_landmark_heavy',
        ]
    elif models_selection == cvtt.ALL_MODELS:
        # preparing all(3) models
        grid = (1, 3)
        display_im_size = (640, 480)
        models_names = tflt.get_all_pose_models_info(ack=False)
    else:
        # preparing 1 models - default is BEST_MODEL
        grid = (1, 1)
        display_im_size = (640, 480)
        models_names = [models_selection]
    models = []
    for m_name in models_names:
        save_dir = '{}/{}/{}'.format(mtt.TFL_MODELS, tflt.MODELS_TFL_META_DATA[OD_BEST_MODEL]['job'], m_name)
        m = tflt.TfltPoseModels(
            save_load_dir=save_dir,
            model_name=m_name,
            tabs=1,
            # threshold=0.2,  # take default
        )
        print(m)
        models.append(m)
    return models, grid, display_im_size


def od_best_model_images_test(m_name: str = OD_BEST_MODEL, ms: int = cvtt.BLOCK_MS_NORMAL):
    mt.get_function_name(ack=True, tabs=0)
    od_models_compare_images_test(
        models_selection=m_name,
        ms=ms
    )
    return


def od_models_compare_images_test(
        cv_imgs_orig: list = None,
        images_names: list = None,
        models_selection: str = cvtt.BEST_MODELS,
        ms: int = cvtt.BLOCK_MS_NORMAL
):
    mt.get_function_name(ack=True, tabs=0)
    models, grid, im_size = __od_get_models(models_selection)

    if cv_imgs_orig is None:
        images_names = [mtt.DOG, mtt.DOGS1, mtt.PERSON]
        cv_imgs_orig = [cvtt.load_img_from_web(image_name) for image_name in images_names]
    else:
        if images_names is None:
            images_names = []
            for i in range(len(cv_imgs_orig)):
                images_names.append('unknown{}'.format(i))
        if len(images_names) != len(cv_imgs_orig):
            mt.exception_error('lists length differ {}, {}'.format(len(images_names), len(cv_imgs_orig)),
                               real_exception=False)
            return

    save_dir = '{}/{}/{}/{}'.format(mtt.IMAGES_OUTPUTS, mt.get_function_name(), SAVE_SCOPE, models_selection)
    mt.create_dir(save_dir)
    cvtt.__od_models_images_test(
        models=models,
        images_names=images_names,
        cv_imgs_orig=cv_imgs_orig,
        grid=grid,
        delay_ms=ms,
        save_dir=save_dir,
        display_im_size=im_size
    )
    return


def od_best_model_video_test(
        m_name: str = OD_BEST_MODEL,
        video_path: str = None,
        video_name: str = 'unknown',
        work: int = 80
):
    mt.get_function_name(ack=True, tabs=0)
    od_models_compare_video_test(
        video_path=video_path,
        video_name=video_name,
        models_selection=m_name,
        work=work
    )
    return


def od_models_compare_video_test(
        video_path: str = None,
        video_name: str = 'unknown',
        work: int = 80,
        models_selection: str = cvtt.BEST_MODELS
):
    mt.get_function_name(ack=True, tabs=0)
    models, grid, im_size = __od_get_models(models_selection)

    if video_path is None:  # default video
        video_name = mtt.DOG1
        video_path = cvtt.get_vid_from_web(name=video_name)

    if not os.path.exists(video_path):
        mt.exception_error(mt.NOT_FOUND.format(video_path), real_exception=False)
        return
    cap = cv2.VideoCapture(video_path)
    if cap.isOpened():
        video_total_frames = cvt.get_frames_from_cap(cap)
        print('\tvid {} has {} frames'.format(video_name, video_total_frames))
        save_dir = '{}/{}/{}/{}'.format(mtt.VIDEOS_OUTPUTS, mt.get_function_name(), SAVE_SCOPE, models_selection)
        mt.create_dir(save_dir)
        cvtt.__od_models_cap_test(
            models=models,
            cap=cap,
            total_frames=video_total_frames,
            work_every_x_frames=work,
            grid=grid,
            delay_ms=1,
            save_dir=save_dir,
            display_im_size=im_size,
            cap_desc=video_name
        )
    else:
        mt.exception_error('cap is closed.', real_exception=False)
    return


def od_best_model_cam_test(
        m_name: str = OD_BEST_MODEL,
        total_frames: int = cvtt.MODEL_FRAMES_CAM
):
    mt.get_function_name(ack=True, tabs=0)
    od_models_compare_cam_test(
        models_selection=m_name,
        total_frames=total_frames
    )
    return


def od_models_compare_cam_test(
        models_selection: str = cvtt.BEST_MODELS,
        total_frames: int = cvtt.MODEL_FRAMES_CAM
):
    mt.get_function_name(ack=True, tabs=0)
    models, grid, im_size = __od_get_models(models_selection)

    cam = cvt.CameraWu.open_camera(port=0, type_cam='cv2')
    if cam is not None:
        save_dir = '{}/{}/{}/{}'.format(mtt.VIDEOS_OUTPUTS, mt.get_function_name(), SAVE_SCOPE, models_selection)
        mt.create_dir(save_dir)
        cvtt.__od_models_cap_test(
            models=models,
            cap=cam,
            total_frames=total_frames,
            work_every_x_frames=1,
            grid=grid,
            delay_ms=1,
            save_dir=save_dir,
            display_im_size=im_size,
            cap_desc='cam 0'
        )
    else:
        mt.exception_error('cap is closed.', real_exception=False)
    return


def pose_best_model_images_test(m_name: str = POSE_BEST_MODEL, ms: int = cvtt.BLOCK_MS_NORMAL):
    mt.get_function_name(ack=True, tabs=0)
    pose_models_compare_images_test(
        models_selection=m_name,
        ms=ms
    )
    return


def pose_models_compare_images_test(
        cv_imgs_orig: list = None,
        images_names: list = None,
        models_selection: str = cvtt.BEST_MODELS,
        ms: int = cvtt.BLOCK_MS_NORMAL
):
    mt.get_function_name(ack=True, tabs=0)
    models, grid, im_size = __pose_get_models(models_selection)

    if cv_imgs_orig is None:
        images_names = [mtt.F_MODEL]
        cv_imgs_orig = [cvtt.load_img_from_web(image_name) for image_name in images_names]
    else:
        if images_names is None:
            images_names = []
            for i in range(len(cv_imgs_orig)):
                images_names.append('unknown{}'.format(i))
        if len(images_names) != len(cv_imgs_orig):
            mt.exception_error('lists length differ {}, {}'.format(len(images_names), len(cv_imgs_orig)),
                               real_exception=False)
            return

    save_dir = '{}/{}/{}/{}'.format(mtt.IMAGES_OUTPUTS, mt.get_function_name(), SAVE_SCOPE, models_selection)
    mt.create_dir(save_dir)
    cvtt.__pose_models_images_test(
        models=models,
        images_names=images_names,
        cv_imgs_orig=cv_imgs_orig,
        grid=grid,
        delay_ms=ms,
        save_dir=save_dir,
        display_im_size=im_size
    )
    return


def test_all():
    print('{}{}:'.format('-' * 5, mt.get_base_file_and_function_name()))
    get_tflite_version_test()
    od_best_model_images_test()
    od_best_model_video_test()
    od_best_model_cam_test()
    od_models_compare_images_test()
    od_models_compare_video_test()
    od_models_compare_cam_test()
    print('{}'.format('-' * 20))
    return
